'use strict';

var debug = require('debug')('bolt');
var constants = require('./constants');
var getBoltObject = require('./getBoltObject');

var socketCallback = function socketCallback(socket) {
  return function (action) {
    var types = constants.types,
        events = constants.events;

    var boltObject = getBoltObject(action);
    if (boltObject) {
      switch (boltObject.event) {
        // Join Channel
        case events.joinChannel:
          debug('\uFE0F\u26A1\uFE0F  Socket %s joined channel "%s"', socket.id, boltObject.channel);
          socket.join(boltObject.channel);
          break;
        // Leave Channel
        case events.leaveChannel:
          debug('\u26A1\uFE0F  Socket %s left channel "%s"', socket.id, boltObject.channel);
          socket.leave(boltObject.channel);
          break;
        // Channel Message
        case events.channelMessage:
          debug('\u26A1\uFE0F  Broadcasting Bolt action "%s" to channel "%s". Action: %o', action.type, boltObject.channel, action);
          socket.broadcast.to(boltObject.channel).emit(events.message, action);
          break;
        // Message
        case events.message:
          debug('\u26A1\uFE0F  Broadcasting Bolt action "%s" to connected sockets. Action: %o', action.type, action);
          socket.broadcast.emit(events.message, action);
          break;
      }
    }
  };
};

/**
 * A callback to handle Redux Bolt actions
 * 
 * @prop socket Socket
 * @type function
 */
var boltCallback = function boltCallback(socket) {
  var events = constants.events;

  for (var event in events) {
    socket.on(events[event], socketCallback(socket));
  }
};

module.exports = boltCallback;